// *******************************************************************
// Copyright (C) 2009 Telos Systems, TLS Corp.
// All rights reserved.
//
// No part of this source code or associated documentation may be 
// reproduced or utilized in any form or by any means, electronic or 
// mechanical, without permission in writing from TLS Corp.
// -------------------------------------------------------------------
//      File: APSend.h
// Description: See class description.
//    Author: Ioan L. Rus
//      Date: Nov 18, 2009
// Revisions: 
// -------------------------------------------------------------------
// INI Date        Description
// --- ----------- ---------------------------------------------------
// 
// *******************************************************************
#include "APSend.h"
#include "Msg2.h"
#include "IAPMessageSender.h"
#include "APMessages.h"

#ifdef WIN32
  #include <winsock2.h>  // for hton#() and ntoh#() functions
  #pragma comment(lib,"ws2_32")
#else
#endif // Win32

// Set this to 1 to print the messages as
// they are decoded.
#define PRINT_SENT_MESSAGES 1

#if PRINT_SENT_MESSAGES
  #include <stdio.h>
  #define TRACE1(x) { printf("<div id=\"function\">"); printf x; printf("</div>\n"); }
#else
  #define TRACE1(x)
#endif


APSend::APSend(IAPMessageSender *pSender)
:
m_pSender(pSender)
{
}

bool APSend::WriteMessage(CMsg2 &msg)
{
  if(m_pSender==0) return false; // No sender

  if(msg.GetDataSize()>(MAX_MESSAGE_SIZE-SIZE_HEADER)) return false; // Message too big. Increase MAX_MESSAGE_SIZE.

  char aMsg[MAX_MESSAGE_SIZE];
  long cbPayload=msg.GetData(aMsg+SIZE_HEADER);

  long lSizePrefix1=htonl(cbPayload);
  long lSizePrefix2=htonl(cbPayload^0xA5A55A5A);

  memcpy(aMsg,&lSizePrefix1,sizeof(long));
  memcpy(aMsg+sizeof(long),&lSizePrefix2,sizeof(long));

  return m_pSender->WriteMessage(aMsg,cbPayload+SIZE_HEADER);
}

bool APSend::Login(const char * pszUser,const char * pszPassword)
{
  TRACE1(("Login(user='%s',password='%s')\n",pszUser,pszPassword));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_CONNECT);

  msg.SetString("USER",pszUser);
  msg.SetString("PASS",pszPassword);

  return WriteMessage(msg);
}

bool APSend::EnumerateShows()
{
  TRACE1(("EnumerateShows()\n"));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_ENUMERATESHOWS);

  return WriteMessage(msg);
}

bool APSend::EnumerateHostsForShow(const char * pszShowName)
{
  TRACE1(("EnumerateHostsForShow(show='%s')\n",pszShowName));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_ENUMERATEHOSTSFORSHOW);

  msg.SetString("SHOW",pszShowName);

  return WriteMessage(msg);
}

bool APSend::EnumerateShowsForHost(const char * pszHostName)
{
  TRACE1(("EnumerateHostsForShow(host='%s')\n",pszHostName));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_ENUMERATESHOWSFORHOST);

  msg.SetString("HOST",pszHostName);

  return WriteMessage(msg);
}

bool APSend::ChangeShow(int nSession,const char * pszShowName,const char * pszShowPass)
{
  TRACE1(("ChangeShow(session=%d,show_name='%s',show_pass='%s')\n",nSession,pszShowName,pszShowPass));
  
  CMsg2 msg;

  msg.SetId(APCLITOSRV_CHANGESHOW);

  msg.SetNumber("SSN_",nSession);
  msg.SetString("SHOW",pszShowName);
  msg.SetString("PASS",pszShowPass);

  return WriteMessage(msg);
}

bool APSend::ConnectToShow(int nSession,const char * pszShowName,const char * pszShowPassword,const char * pszHostName)
{
  TRACE1(("ConnectToShow(session=%d,show_name='%s',show_pass='%s',host='%s')\n",nSession,pszShowName,pszShowPassword,pszHostName));
  
  CMsg2 msg;

  msg.SetId(APCLITOSRV_CONNECTTOSHOW);

  msg.SetNumber("SSN_",nSession);
  msg.SetString("SHOW",pszShowName);
  msg.SetString("PASS",pszShowPassword);
  msg.SetString("HOST",pszHostName);

  return WriteMessage(msg);
}

bool APSend::EnumerateDirectors(int nSession)
{
  TRACE1(("EnumerateDirectors(session=%d)\n",nSession));
  
  CMsg2 msg;

  msg.SetId(APCLITOSRV_ENUMERATEDIRECTORS);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::AttachToDirector(int nSession,const char * pszDirectorName)
{
  TRACE1(("AttachToDirector(session=%d,dir_name='%s')\n",nSession,pszDirectorName));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_ATTACHTODIRECTOR);

  msg.SetNumber("SSN_",nSession);
  msg.SetString("DNAM",pszDirectorName);

  return WriteMessage(msg);
}

bool APSend::AttachAsTalent(int nSession)
{
  TRACE1(("AttachAsTalent(session=%d)\n",nSession));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_ATTACHASTALENT);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::PostBusyAllButton(int nSession)
{
  TRACE1(("PostBusyAllButton(session=%d)\n",nSession));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTBUSYALLBUTTON);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::PostDelayDumpButton(int nSession,bool bDown)
{
  TRACE1(("PostDelayDumpButton(session=%d,down=%s)\n",nSession,bDown?"true":"false"));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTDELAYDUMPBUTTON);

  msg.SetNumber("SSN_",nSession);
  msg.SetNumber("DOWN",bDown);

  return WriteMessage(msg);
}

bool APSend::PostDialKey(int nSession,int chDialKey)
{
  TRACE1(("PostDialKey(session=%d,key=%d)\n",nSession,chDialKey));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTDIALKEY);

  msg.SetNumber("SSN_",nSession);
  msg.SetNumber("KEY_",chDialKey);

  return WriteMessage(msg);
}

bool APSend::PostDropButton(int nSession,int nColumn)
{
  TRACE1(("PostDropButton(session=%d,column=%d)\n",nSession,nColumn));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTDROPBUTTON);

  msg.SetNumber("SSN_",nSession);
  msg.SetNumber("COL_",nColumn);

  return WriteMessage(msg);
}

bool APSend::PostHoldButton(int nSession,int nColumn)
{
  TRACE1(("PostHoldButton(session=%d,column=%d)\n",nSession,nColumn));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTHOLDBUTTON);

  msg.SetNumber("SSN_",nSession);
  msg.SetNumber("COL_",nColumn);

  return WriteMessage(msg);
}

bool APSend::PostLineButton(int nSession,int nLine,int nColumn)
{
  TRACE1(("PostLineButton(session=%d,line=%d,column=%d)\n",nSession,nLine,nColumn));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTLINEBUTTON);

  msg.SetNumber("SSN_",nSession);
  msg.SetNumber("LINE",nLine);
  msg.SetNumber("COL_",nColumn);

  return WriteMessage(msg);
}

bool APSend::PostMuteButton(int nSession)
{
  TRACE1(("PostMuteButton(session=%d)\n",nSession));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTMUTEBUTTON);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::PostNextButton(int nSession)
{
  TRACE1(("PostNextButton(session=%d)\n",nSession));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTNEXTBUTTON);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::PostRecordButton(int nSession)
{
  TRACE1(("PostRecordButton(session=%d)\n",nSession));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTRECORDBUTTON);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::PostSpeakerButton(int nSession)
{
  TRACE1(("PostSpeakerButton(session=%d)\n",nSession));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTSPEAKERBUTTON);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::PostText(int nSession,int nTextType,int nLine,const char * pszText)
{
  TRACE1(("PostText(session=%d,type=%d,line=%d,text='%s')\n",nSession,nTextType,nLine,pszText));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTTEXT);

  msg.SetNumber("SSN_",nSession);
  msg.SetNumber("TTYP",nTextType);
  msg.SetNumber("LINE",nLine);
  msg.SetString("TEXT",pszText);

  return WriteMessage(msg);
}

bool APSend::PostTransferButton(int nSession)
{
  TRACE1(("PostTransferButton(session=%d)\n",nSession));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTTRANSFERBUTTON);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::PostUpdateAll(int nSession)
{
  TRACE1(("PostUpdateAll(session=%d)\n",nSession));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTUPDATEALL);

  msg.SetNumber("SSN_",nSession);

  return WriteMessage(msg);
}

bool APSend::PostMuteRinger(int nSession,int bOn)
{
  TRACE1(("PostMuteRinger(session=%d,on=%s)\n",nSession,bOn!=0?"true":"false"));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTMUTERINGER);

  msg.SetNumber("SSN_",nSession);
  msg.SetNumber("ON__",bOn);

  return WriteMessage(msg);
}

bool APSend::PostAcousticAdapt(int nSession,int bOn)
{
  TRACE1(("PostAcousticAdapt(session=%d,on=%s)\n",nSession,bOn!=0?"true":"false"));

  CMsg2 msg;

  msg.SetId(APCLITOSRV_POSTACOUSTICADAPT);

  msg.SetNumber("SSN_",nSession);
  msg.SetNumber("ON__",bOn);

  return WriteMessage(msg);
}

