// *******************************************************************
// Copyright (C) 2009 Telos Systems, TLS Corp.
// All rights reserved.
//
// No part of this source code or associated documentation may be 
// reproduced or utilized in any form or by any means, electronic or 
// mechanical, without permission in writing from TLS Corp.
// -------------------------------------------------------------------
//      File: APSend.h
// Description: See class description.
//    Author: Ioan L. Rus
//      Date: Nov 18, 2009
// Revisions: 
// -------------------------------------------------------------------
// INI Date        Description
// --- ----------- ---------------------------------------------------
// 
// *******************************************************************
#ifndef _APSEND_H_INCLUDED_
#define _APSEND_H_INCLUDED_

class IAPMessageSender;

class CMsg2;

// Creates a message packet to be sent to the AP server each
// time one of the server functions is invoked.
class APSend
{
private:
  IAPMessageSender *m_pSender;

  enum { MAX_MESSAGE_SIZE=1024, SIZE_HEADER=8 };
  bool WriteMessage(CMsg2 &msg);

public:
  // The client application needs to provide an IAPMessageSender
  // pointer. IAPMessageSender will be responsible for sending
  // the packet to the AP server.
  APSend(IAPMessageSender *pSender);

  // Connect/Disconnect to/from server
  bool Login(const char *pszUser,const char *pszPassword);

  // Get a list of shows
  bool EnumerateShows();
  
  // Get a list of hosts where the specified show is allowed to run
  bool EnumerateHostsForShow(const char *pszShowName);
  
  // Get a list of shows allowed to run on specified host
  bool EnumerateShowsForHost(const char * pszHostName);

  // Connect to existing show
  bool ConnectToShow(int nSession,const char *pszShowName,const char *pszShowPassword,const char *pszHostName);
  
  // Create a new show
  bool ChangeShow(int nSession,const char *pszShowName,const char *pszShowPass);
  
  // Get a list of directors attached to a show
  bool EnumerateDirectors(int nSession);
  
  // Attach to existing director
  bool AttachToDirector(int nSession,const char *pszDirectorName);
  
  // Run stand alone (always in Talent mode)
  bool AttachAsTalent(int nSession);

  // Methods that can be invoked after attach
  bool PostBusyAllButton(int nSession);
  bool PostDelayDumpButton(int nSession,bool bDown);
  bool PostDialKey(int nSession,int chDialKey);
  bool PostDropButton(int nSession,int nColumn);
  bool PostHoldButton(int nSession,int nColumn);
  bool PostLineButton(int nSession,int nLine,int nColumn);
  bool PostMuteButton(int nSession);
  bool PostNextButton(int nSession);
  bool PostRecordButton(int nSession);
  bool PostSpeakerButton(int nSession);
  bool PostText(int nSession,int nTextType,int nLine,const char *pszText);
  bool PostTransferButton(int nSession);
  bool PostUpdateAll(int nSession);
  bool PostMuteRinger(int nSession,int bOn);
  bool PostAcousticAdapt(int nSession,int bOn);
};

#endif // _APSEND_H_INCLUDED_
